//: ARRAY1.CPP -- Array template with iterator.
//. Actual objects, not just pointers to objects,
//. can be easily stored with templates.
//. Adapted from "Using C++" 2nd edition, Osborne/McGraw-Hill 1992
//. by Bruce Eckel
#include <iostream.h>
#include "ostring.h"

template <class T, int size> class arrayIter;

template<class T, int size> class array {
  T a[size];
public:
  T& operator[](int i) {  // for assignment
    if(i >= size || i < 0) {
      cerr << "array exceeded" << endl;
      return *new T; // dummy value
    }
    return a[i];
  }
  friend class arrayIter<T, size>;
};

template<class T, int size>
class arrayIter {
  static T dummy;
  array<T,size>& ar;
  int index;
public:
  arrayIter(array<T, size>& A)
      : ar(A), index(0) {}
  T operator++() {  // pre-increment
    if(++index >= size) return dummy;
    return ar.a[index]; // a copy
  }
  T operator++(int) {  // post-increment
    if(index >= size) return dummy;
    return ar.a[index++]; // a copy
  }
  operator T() {
    if(index >= size) return dummy;
    return ar.a[index]; // a copy
  }
  int end() { return index >= size; }
};

// Defining a template static data member:
template<class T, int size>
  T arrayIter<T,size>::dummy(0);

void main() {
  array<String, 5> S;
  S[0] = "are";
  S[1] = "we";
  S[2] = "having";
  S[3] = "fun";
  S[4] = "yet?";
  S[5] = "too much!"; // exceeds array
  arrayIter<String, 5> si(S);
  while(!si.end())
    cout << si++ << " ";
}
